#!/usr/bin/perl
package dummyplug;

# Name: IAM, Interactive Account Manager
# Description: Multi server Multi domain account manager
# Version: 0.8
# Authors: Jason Jorgensen <jasonj@innominatus.com>
# Copyright (C) 2003  Jason Jorgensen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###########################################################################



# This is dummy plug, basically returns true for everything

#use strict;

BEGIN {
    require Exporter;
    use vars qw( $VERSION @ISA );
	$VERSION     = (split ' ', q!$Revision: 0.6 $!)[1];
    @ISA          =  qw( Exporter );
}

sub new {
  my $class = shift @_;
  my $self = {};
  
  bless ( $self, $class );
}

sub close {
  my $self = shift @_;

}

sub debug {
  my $self = shift @_;

  &check_for_errors($@);
  $self->{'debug'} = 1;
}

sub activate_fields {
  my $self = shift @_;

  my $requiredfields = "name=>uniqueid,desc=>Unique Identifier,minlen=>0,maxlen=>64,optional=>0                        name=>username,desc=>Username,minlen=>4,maxlen=>64,optional=>0";

  return "0 ".$requiredfields;
}

sub activate {
  my $self = shift @_;
  my $uniqueid = shift @_;
  my $username = shift @_;

  my @errors;

  if (! $uniqueid) { push @errors, 100; }
  if (! $username) { push @errors, 101; }
  if (! $currentpassword) { push @errors, 300; }

  if ($#errors < 0) {
    return 0;
  }
  else {
    return &display_errors($self, \@errors);
  }
}

sub add_fields {
  my $self = shift @_;
  
  my $requiredfields = "name=>uniqueid,desc=>Unique Identifier,minlen=>0,maxlen=>64,optional=>0
                        name=>username,desc=>Username,minlen=>4,maxlen=>64,optional=>0
                        name=>password,desc=>User Password,minlen=>4,maxlen=>64,optional=>0
                        name=>cryptedpassword,desc=>Already Crypted User Password,minlen=>4,maxlen=>64,optional=>1\n";
  return "0 ".$requiredfields;
}

sub add {
  my $self = shift @_;
  my $uniqueid = shift @_;
  my $username = shift @_;
  my $password = shift @_;
  my $cryptedpassword = shift @_;

  my @errors;
  
  
  if (! $uniqueid) { push @errors, 100; }
  if (! $username) { push @errors, 101; }
  if (! $password && ! $cryptedpassword) { push @errors, 102; }

  if ($#errors < 0) {
    return 0;
  }
  else {
    return &display_errors($self, \@errors);
  }
}

sub deactivate_fields {
  my $self = shift @_;

  my $requiredfields = "name=>uniqueid,desc=>Unique Identifier,minlen=>0,maxlen=>64,optional=>0                        name=>username,desc=>Username,minlen=>4,maxlen=>64,optional=>0";

  return "0 ".$requiredfields;
}

sub deactivate {
  my $self = shift @_;
  my $uniqueid = shift @_;
  my $username = shift @_;

  my @errors;

  if (! $uniqueid) { push @errors, 100; }
  if (! $username) { push @errors, 101; }
  if (! $currentpassword) { push @errors, 300; }

  if ($#errors < 0) {
    return 0;
  }
  else {
    return &display_errors($self, \@errors);
  }
}

sub modify_fields {
  my $self = shift @_;
  
  my $requiredfields = "name=>uniqueid,desc=>Unique Identifier,minlen=>0,maxlen=>64,optional=>0
                        name=>username,desc=>Username,minlen=>4,maxlen=>64,optional=>0
                        name=>newuniqueid,desc=>New Unique Identifier,minlen=>0,maxlen=>64,optional=>1
                        name=>newusername,desc=>New Username,minlen=>4,maxlen=>64,optional=>1
                        name=>newpassword,desc=>New User Password,minlen=>4,maxlen=>64,optional=>1
                        name=>newcryptedpassword,desc=>New Already Crypted User Password,minlen=>4,maxlen=>64,optional=>1\n";
  return "0 ".$requiredfields;
}

sub modify {
  my $self = shift @_;
  my $uniqueid = shift @_;
  my $username = shift @_;
  my $newuniqueid = shift @_;
  my $newusername = shift @_;
  my $newpassword = shift @_;
  my $newcryptedpassword = shift @_;

  my @errors;
  
  $rand = new String::Random;
  $salt = $rand->randpattern("ssssssss");

  if (! $uniqueid) { push @errors, 100; }
  if (! $username) { push @errors, 101; }

  if ($#errors < 0) {
    return 0;
  }
  else {
    return &display_errors($self, \@errors);
  }
}

sub delete_fields {
  my $self = shift @_;
  
  my $requiredfields = "name=>uniqueid,desc=>Unique Identifier,minlen=>0,maxlen=>64,optional=>1
                        name=>username,desc=>Username,minlen=>4,maxlen=>64,optional=>1
                        name=>password,desc=>User Password,minlen=>4,maxlen=>64,optional=>1\n";
  return "0 ".$requiredfields;
}

sub delete {
  my $self = shift @_;
  my $uniqueid = shift @_;
  my $username = shift @_;
  
  my @errors;

  if (! $uniqueid) { push @errors, 100; }
  if (! $username) { push @errors, 101; }

  if ($#errors < 0) {
    return 0;
  }
  else {
    return &display_errors($self, \@errors);
  }
  
}

sub display {
  my $self = shift @_;
  my $uniqueid = shift @_;
  my $username = shift @_;
  
  my @errors;

  if (! $uniqueid) { push @errors, 100; }
  if (! $username) { push @errors, 101; }

  if ($#errors < 0) {
    return "0 ";
  }
  else {
    return &display_errors($self, \@errors);
  }
}

sub check_for_errors {
  my $self = shift @_;
  $error = shift @_;

  if ($error) {
    print STDERR "Error: $error\n";
    return "1 Unknown error occured, inform administrator\n";
  }                                                                                                                                        
}

sub display_errors {
  my $self = shift @_;
  my @errors = @{shift @_};
  my $errmsg;

  foreach $error (@errors) {
    $errmsg .= "$error ";
    if ($error == 666) { print "Yet to be classified error"; }
    
    elsif ($error == 100) { $errmsg .= "Must specify a 'uniqueid'"; }
    elsif ($error == 101) { $errmsg .= "Must specify a 'username'"; }
    elsif ($error == 102) { $errmsg .= "Must specify a 'password'"; }
    
    elsif ($error == 200) { $errmsg .= "Duplicate Username!"; }
    elsif ($error == 201) { $errmsg .= "Username doesnt exist!"; }

    elsif ($error == 1501) { $errmsg .= "Must specify a type when calling function 'singledb'"; }
    
    else { $errmsg .= "Unknown error"; }
    $errmsg .= "\n";
  }
  return $errmsg;
}


1;

__END__