#!/bin/sh

#######################################################################################################
# Edit this stuff to match your location
COUNTRY="US"                         # Country                             - US, AU, UK, CA
STATE="ST"                           # State                               - MN, IA, CA, NY
LOCALITY="CITY"                      # City, Town, Village                 - Minneapolis, Mexico City,
ORGANIZATION="ORGANIZATION"     # Organization, Company, Affiliate    - Joe's Gas Station, The
UNIT="DEPARTMENT"         # Organizational unit, Department     - System Adminis
COMMONNAME=`hostname --fqdn`         # Common Name, Host name of server    - fully.qualified.do
EMAIL="SYSADMIN EMAIL HERE"             # Email address of authority for cert - sysadmin@bar.com, t
#######################################################################################################

TAR=`which tar`
DATE=`which date`
OPENSSL=`which openssl`
CURRENTDATE=`$DATE -u +%m.%d.%Y`
HOSTS="IAMd-client IAMd-server"


ANSWERS="$COUNTRY\n$STATE\n$LOCALITY\n$ORGANIZATION\n$UNIT\n$COMMONNAME\n$EMAIL\n\n\n"


for VIRTUALHOST in $HOSTS
do
  if [ -e "$VIRTUALHOST.key" ] || [ -e "$VIRTUALHOST.csr" ] || [ -e "$VIRTUALHOST.crt" ] || [ -e "$VIRTUALHOST.rand" ]
  then
    echo "*** WARNING ***"
    echo "  IAM certificates already exist!"
    echo "  I will not overwrite them, move them to a safe location, or delete them first"
    exit 0
  fi
  
  echo "*************************************"
  echo "Creating Certificate for $VIRTUALHOST"
  echo "*************************************"
  echo
  
  echo "*** Creating random file for use with openssl ***"
  dd if=/dev/urandom of=$VIRTUALHOST.rand bs=2048 count=1
  
  echo "*** This command will generate 2048 bit RSA Private Key and stores it in the file $VIRTUALHOST.key.  ***"
  echo "*** Protect this key with your life, dont let it fall into the wrong hands.                          ***"
  $OPENSSL genrsa -rand $VIRTUALHOST.rand -out $VIRTUALHOST.key 2048
  
  
  echo "*** Enter the FQDN of the server your making the cert for in the common name spot ***"
  echo -e $ANSWERS | $OPENSSL req -new -key $VIRTUALHOST.key -out $VIRTUALHOST.csr
  
  
  echo "*** This command will generate a certificate a self-signed certificate in $VIRTUALHOST.crt ***"
  $OPENSSL x509 -req -days 3650 -in $VIRTUALHOST.csr -signkey $VIRTUALHOST.key -out $VIRTUALHOST.crt   
  
    
  if [ -e "$VIRTUALHOST.key" ] && [ -e "$VIRTUALHOST.csr" ] && [ -e "$VIRTUALHOST.crt" ] && [ -e "$VIRTUALHOST.rand" ]
  then
    echo "*** Creation successful! ***"
    tar zcvf $VIRTUALHOST-$CURRENTDATE.tar.gz $VIRTUALHOST.rand $VIRTUALHOST.key $VIRTUALHOST.crt $VIRTUALHOST.csr >/dev/null
    echo "*** BACKUP $VIRTUALHOST-$CURRENTDATE.tar.gz to a floppy disk and store it in a secure place***"
  else
    echo "*** Creation FAILED, you will have to troubleshoot ***"
  fi
done